@@ -10,6 +10,7 @@ from InquirerPy import get_style, inquirer
 from rich.console import Console
 
 from devcommit.app.gemini_ai import generateCommitMessage
+from devcommit.app.changelog import generate_changelog, save_changelog
 from devcommit.utils.git import (KnownError, assert_git_repo,
                                  get_detected_message, get_diff_for_files,
                                  get_files_from_paths, get_staged_diff,
@@ -34,6 +35,36 @@ def has_commits() -> bool:
     return result.returncode == 0
 
 
+def sanitize_commit_messages(messages):
+    """Filter out error messages from commit message list.
+    
+    Args:
+        messages: String or list of commit messages
+        
+    Returns:
+        List of valid commit messages with error messages filtered out
+    """
+    # Convert string to list if needed
+    if isinstance(messages, str):
+        if not messages or messages.strip() == "":
+            return []
+        messages = messages.split("|")
+    
+    # Filter out error messages and empty strings
+    valid_messages = []
+    for msg in messages:
+        if not msg:
+            continue
+        msg = msg.strip()
+        # Skip if it's an error message
+        if msg.startswith("Error generating commit message:"):
+            continue
+        if msg and msg.strip():
+            valid_messages.append(msg)
+    
+    return valid_messages
+
+
 # Main function
 def main(flags: CommitFlag = None):
     if flags is None:
@@ -99,6 +130,42 @@ def main(flags: CommitFlag = None):
             except Exception as e:
                 raise KnownError(f"Failed to get files from paths: {str(e)}")
 
+
+        # Generate changelog before staging if --changelog and --stageAll are both set
+        if flags["stageAll"] and flags["changelog"]:
+            console.print()
+            console.print("[bold cyan]📝 Generating changelog...[/bold cyan]")
+            
+            with console.status(
+                "[magenta]🤖 AI generating changelog from changes...[/magenta]",
+                spinner="dots",
+                spinner_style="magenta"
+            ):
+                # Get diff of unstaged changes
+                if push_files_list:
+                    # Get diff for specific files
+                    diff = get_diff_for_files(push_files_list, flags["excludeFiles"])
+                else:
+                    # Get diff for all changes
+                    import subprocess
+                    diff = subprocess.run(
+                        ["git", "diff"],
+                        stdout=subprocess.PIPE,
+                        text=True,
+                    ).stdout
+                
+                if diff:
+                    try:
+                        changelog_content = generate_changelog(diff)
+                        changelog_path = save_changelog(changelog_content)
+                        console.print(f"[bold green]✅ Changelog saved to:[/bold green] [cyan]{changelog_path}[/cyan]")
+                    except Exception as e:
+                        logger.error(f"Failed to generate changelog: {e}")
+                        console.print(f"[bold yellow]⚠️  Failed to generate changelog: {e}[/bold yellow]")
+                else:
+                    console.print("[bold yellow]⚠️  No changes to generate changelog from[/bold yellow]")
+            console.print()
+
         if flags["stageAll"]:
             if push_files_list:
                 # Stage specific files/folders only
@@ -258,6 +325,36 @@ def main(flags: CommitFlag = None):
         elif flags.get("push", False) and not commit_made:
             console.print("\n[bold yellow]⚠️  No commits were made, skipping push[/bold yellow]\n")
         
+        # Generate changelog after commit if --changelog is used without --stageAll
+        if flags.get("changelog", False) and not flags.get("stageAll", False) and commit_made:
+            console.print()
+            console.print("[bold cyan]📝 Generating changelog from committed changes...[/bold cyan]")
+            
+            with console.status(
+                "[magenta]🤖 AI generating changelog...[/magenta]",
+                spinner="dots",
+                spinner_style="magenta"
+            ):
+                # Get diff from last commit
+                import subprocess
+                diff = subprocess.run(
+                    ["git", "diff", "HEAD~1", "HEAD"],
+                    stdout=subprocess.PIPE,
+                    text=True,
+                ).stdout
+                
+                if diff:
+                    try:
+                        changelog_content = generate_changelog(diff)
+                        changelog_path = save_changelog(changelog_content)
+                        console.print(f"[bold green]✅ Changelog saved to:[/bold green] [cyan]{changelog_path}[/cyan]")
+                    except Exception as e:
+                        logger.error(f"Failed to generate changelog: {e}")
+                        console.print(f"[bold yellow]⚠️  Failed to generate changelog: {e}[/bold yellow]")
+                else:
+                    console.print("[bold yellow]⚠️  No changes to generate changelog from[/bold yellow]")
+            console.print()
+        
         # Print stylish completion message only if commits were made
         if commit_made:
             console.print()
@@ -273,7 +370,8 @@ def main(flags: CommitFlag = None):
         return
     except KnownError as error:
         logger.error(str(error))
-        console.print(f"\n[bold red]❌ Error:[/bold red] [red]{error}[/red]\n")
+        # Don't print here - Rich status context already displays the error
+        pass
     except subprocess.CalledProcessError as error:
         logger.error(str(error))
         console.print(f"\n[bold red]❌ Git command failed:[/bold red] [red]{error}[/red]\n")
@@ -322,6 +420,9 @@ def analyze_changes(console, files=None):
     """
     import sys
     
+    # Store any exception to re-raise after status context exits
+    caught_exception = None
+    
     with console.status(
         "[magenta]🤖 AI analyzing changes...[/magenta]",
         spinner="dots",
@@ -350,17 +451,28 @@ def analyze_changes(console, files=None):
         
         try:
             commit_message = generateCommitMessage(diff)
+        except KnownError as e:
+            # Catch KnownError to prevent Rich status from printing it
+            caught_exception = e
+            commit_message = None
         finally:
             sys.stderr = _stderr
             _devnull.close()
         
-        if isinstance(commit_message, str):
-            commit_message = commit_message.split("|")
+        # If we caught an exception, we'll re-raise it after the status context exits
+        if caught_exception:
+            pass  # Will be raised below
+        else:
+            commit_message = sanitize_commit_messages(commit_message)
 
-        if not commit_message:
-            raise KnownError("No commit messages were generated. Try again.")
-
-        return commit_message
+            if not commit_message:
+                raise KnownError("No commit messages were generated. Try again.")
+    
+    # Re-raise the exception outside the status context to avoid duplicate printing
+    if caught_exception:
+        raise caught_exception
+    
+    return commit_message
 
 
 def prompt_commit_message(console, commit_message, regenerate_callback=None):
@@ -703,8 +815,7 @@ def process_per_directory_commits(console, staged, flags):
                 sys.stderr = _stderr
                 _devnull.close()
             
-            if isinstance(commit_message, str):
-                commit_message = commit_message.split("|")
+            commit_message = sanitize_commit_messages(commit_message)
             
             if not commit_message:
                 console.print(f"\n[bold yellow]⚠️  No commit message generated for {directory}, skipping[/bold yellow]\n")
@@ -722,9 +833,7 @@ def process_per_directory_commits(console, staged, flags):
                 sys.stderr = _devnull
                 try:
                     msg = generateCommitMessage(diff)
-                    if isinstance(msg, str):
-                        msg = msg.split("|")
-                    return msg
+                    return sanitize_commit_messages(msg)
                 finally:
                     sys.stderr = _stderr
                     _devnull.close()
@@ -865,8 +974,7 @@ def process_per_file_commits(console, staged, flags):
                 sys.stderr = _stderr
                 _devnull.close()
             
-            if isinstance(commit_message, str):
-                commit_message = commit_message.split("|")
+            commit_message = sanitize_commit_messages(commit_message)
             
             if not commit_message:
                 console.print(f"\n[bold yellow]⚠️  No commit message generated for {file}, skipping[/bold yellow]\n")
@@ -884,9 +992,7 @@ def process_per_file_commits(console, staged, flags):
                 sys.stderr = _devnull
                 try:
                     msg = generateCommitMessage(diff)
-                    if isinstance(msg, str):
-                        msg = msg.split("|")
-                    return msg
+                    return sanitize_commit_messages(msg)
                 finally:
                     sys.stderr = _stderr
                     _devnull.close()
@@ -997,8 +1103,7 @@ def process_per_directory_commits_from_paths(console, staged, flags, original_pa
                 sys.stderr = _stderr
                 _devnull.close()
             
-            if isinstance(commit_message, str):
-                commit_message = commit_message.split("|")
+            commit_message = sanitize_commit_messages(commit_message)
             
             if not commit_message:
                 console.print(f"\n[bold yellow]⚠️  No commit message generated for {path}, skipping[/bold yellow]\n")
@@ -1016,9 +1121,7 @@ def process_per_directory_commits_from_paths(console, staged, flags, original_pa
                 sys.stderr = _devnull
                 try:
                     msg = generateCommitMessage(diff)
-                    if isinstance(msg, str):
-                        msg = msg.split("|")
-                    return msg
+                    return sanitize_commit_messages(msg)
                 finally:
                     sys.stderr = _stderr
                     _devnull.close()
@@ -1140,7 +1243,7 @@ def process_per_related_commits(console, staged, flags):
             # Fallback mode detected - AI parsing failed, so commit_messages are empty
             # Generate commit messages for all groups (one call per group, but done upfront)
             # This is still extra calls, but at least they're done before user interaction
-            from devcommit.app.gemini_ai import generateCommitMessage
+            # Note: generateCommitMessage is already imported at the top of the file
             
             console.print("[dim]⚠️  AI grouping response had no commit messages. Generating now...[/dim]")
             with console.status(
@@ -1160,9 +1263,7 @@ def process_per_related_commits(console, staged, flags):
                         diff = get_diff_for_files(group_files, flags["excludeFiles"])
                         if diff:
                             commit_msgs = generateCommitMessage(diff)
-                            if isinstance(commit_msgs, str):
-                                commit_msgs = commit_msgs.split("|")
-                            group_data['commit_messages'] = [msg.strip() for msg in commit_msgs if msg and msg.strip()]
+                            group_data['commit_messages'] = sanitize_commit_messages(commit_msgs)
                 finally:
                     sys.stderr = _stderr
                     _devnull.close()
@@ -1345,8 +1446,7 @@ def process_per_related_commits(console, staged, flags):
                     sys.stderr = _stderr
                     _devnull.close()
                 
-                if isinstance(commit_message, str):
-                    commit_message = commit_message.split("|")
+                commit_message = sanitize_commit_messages(commit_message)
                 
                 if not commit_message:
                     console.print(f"\n[bold yellow]⚠️  No commit message generated for {group_name}, skipping[/bold yellow]\n")
@@ -1364,9 +1464,7 @@ def process_per_related_commits(console, staged, flags):
                 sys.stderr = _devnull
                 try:
                     msg = generateCommitMessage(diff)
-                    if isinstance(msg, str):
-                        msg = msg.split("|")
-                    return msg
+                    return sanitize_commit_messages(msg)
                 finally:
                     sys.stderr = _stderr
                     _devnull.close()
